@@ -90,6 +90,24 @@ impl Config {
     }
 }
 
+#[derive(Clone, Copy, Debug, serde::Deserialize, serde::Serialize)]
+pub enum Direction {
+    #[serde(rename = "up")]
+    Up,
+
+    #[serde(rename = "down")]
+    Down,
+}
+
+impl std::fmt::Display for Direction {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Up => write!(f, "up"),
+            Self::Down => write!(f, "down"),
+        }
+    }
+}
+
 #[derive(Clone)]
 pub struct State {
     upstream: Url,
@@ -161,6 +179,19 @@ impl State {
         ))
     }
 
+    fn move_entry_path(
+        &self,
+        collection_id: Uuid,
+        id: Uuid,
+        token: &ValidToken,
+        direction: Direction,
+    ) -> String {
+        self.scoped(&format!(
+            "{}/entry/{}/move/{}?token={}",
+            collection_id, id, direction, token.token
+        ))
+    }
+
     fn delete_entry_path(
         &self,
         collection_id: Uuid,
@@ -247,6 +278,9 @@ pub fn configure(cfg: &mut web::ServiceConfig, state: State, client: Client) {
                     .service(
                         web::scope("/{entry}")
                             .service(web::resource("").route(web::post().to(update_entry)))
+                            .service(
+                                web::resource("/move/{direction}").route(web::get().to(move_entry)),
+                            )
                             .service(web::resource("/delete").route(web::get().to(delete_entry))),
                     ),
             ),
@@ -448,7 +482,11 @@ impl CollectionPath {
         format!("{}", self.collection)
     }
 
-    fn entry_range(&self) -> std::ops::Range<Vec<u8>> {
+    fn order_key(&self) -> String {
+        format!("{}/order", self.collection)
+    }
+
+    fn entry_range(&self) -> std::ops::RangeInclusive<Vec<u8>> {
         let base = format!("{}/entry/", self.collection).as_bytes().to_vec();
         let mut start = base.clone();
         let mut end = base;
@@ -456,7 +494,7 @@ impl CollectionPath {
         start.push(0x0);
         end.push(0xff);
 
-        start..end
+        start..=end
     }
 
     fn token_key(&self) -> String {
@@ -470,12 +508,36 @@ struct EntryPath {
     entry: Uuid,
 }
 
+#[derive(Debug, serde::Deserialize)]
+struct MoveEntryPath {
+    collection: Uuid,
+    entry: Uuid,
+    direction: Direction,
+}
+
 impl EntryPath {
     fn key(&self) -> String {
         format!("{}/entry/{}", self.collection, self.entry)
     }
 }
 
+impl MoveEntryPath {
+    fn order_key(&self) -> String {
+        format!("{}/order", self.collection)
+    }
+
+    fn entry_range(&self) -> std::ops::RangeInclusive<Vec<u8>> {
+        let base = format!("{}/entry/", self.collection).as_bytes().to_vec();
+        let mut start = base.clone();
+        let mut end = base;
+
+        start.push(0x0);
+        end.push(0xff);
+
+        start..=end
+    }
+}
+
 #[tracing::instrument(name = "Upload image", skip(pl))]
 async fn upload(
     req: HttpRequest,
@@ -588,7 +650,10 @@ async fn view_collection(
         Some(collection) => collection,
         None => return Ok(to_404(&state)),
     };
-    let entries = state.store.entries(path.entry_range()).await?;
+    let entries = state
+        .store
+        .entries(path.order_key(), path.entry_range())
+        .await?;
 
     rendered(
         |cursor| {
@@ -608,7 +673,10 @@ async fn edit_collection(
         Some(collection) => collection,
         None => return Ok(to_404(&state)),
     };
-    let entries = state.store.entries(path.entry_range()).await?;
+    let entries = state
+        .store
+        .entries(path.order_key(), path.entry_range())
+        .await?;
 
     rendered(
         |cursor| {
@@ -672,6 +740,21 @@ async fn update_collection(
     Ok(to_edit_page(path.collection, &token, &state))
 }
 
+async fn move_entry(
+    path: web::Path<MoveEntryPath>,
+    token: ValidToken,
+    state: web::Data<State>,
+) -> Result<HttpResponse, StateError> {
+    store::MoveEntry {
+        move_entry_path: &path,
+    }
+    .exec(&state.store)
+    .await
+    .stateful(&state)?;
+
+    Ok(to_edit_page(path.collection, &token, &state))
+}
+
 #[tracing::instrument(name = "Update Entry")]
 async fn update_entry(
     entry_path: web::Path<EntryPath>,
@@ -774,7 +857,7 @@ async fn delete_collection(
 
     let entries = state
         .store
-        .entries(path.entry_range())
+        .entries(path.order_key(), path.entry_range())
         .await
         .stateful(&state)?;
 
